@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2016 Aleix Pol Gonzalez <aleixpol@kde.org>
+ * Copyright (C) 2018 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -44,6 +45,11 @@ public:
         return rd.m_release == m_release;
     }
 
+    AsRelease *release() const
+    {
+        return m_release;
+    }
+
     AsRelease* m_release;
 };
 
@@ -68,6 +74,16 @@ bool Release::operator==(const Release &other) const
     return false;
 }
 
+_AsRelease * AppStream::Release::asRelease() const
+{
+    return d->release();
+}
+
+Release::Kind Release::kind() const
+{
+    return Release::Kind(as_release_get_kind(d->m_release));
+}
+
 QString Release::version() const
 {
     return QString::fromUtf8(as_release_get_version(d->m_release));
