@@ -21,29 +21,14 @@
 #ifndef __AS_COMPONENT_PRIVATE_H
 #define __AS_COMPONENT_PRIVATE_H
 
-#include <glib-object.h>
 #include "as-component.h"
 #include "as-settings-private.h"
+#include "as-xml.h"
+#include "as-yaml.h"
 
 G_BEGIN_DECLS
 #pragma GCC visibility push(hidden)
 
-/**
- * AsComponentScope:
- * @AS_COMPONENT_SCOPE_UNKNOWN:		Unknown scope
- * @AS_COMPONENT_SCOPE_USER:		User scope
- * @AS_COMPONENT_SCOPE_SYSTEM:		System scope
- *
- * Scope of the #AsComponent (system-wide or user-scope)
- **/
-typedef enum {
-	AS_COMPONENT_SCOPE_UNKNOWN,
-	AS_COMPONENT_SCOPE_USER,
-	AS_COMPONENT_SCOPE_SYSTEM,
-	/*< private >*/
-	AS_COMPONENT_SCOPE_LAST
-} AsComponentScope;
-
 /**
  * AsOriginKind:
  * @AS_ORIGIN_KIND_UNKNOWN:		Unknown origin kind.
@@ -77,20 +62,12 @@ void			as_component_complete (AsComponent *cpt,
 						gchar *scr_base_url,
 						GPtrArray *icon_paths);
 
-GHashTable		*as_component_get_name_table (AsComponent *cpt);
-GHashTable		*as_component_get_summary_table (AsComponent *cpt);
-GHashTable		*as_component_get_description_table (AsComponent *cpt);
-GHashTable		*as_component_get_developer_name_table (AsComponent *cpt);
-GHashTable		*as_component_get_keywords_table (AsComponent *cpt);
 AS_INTERNAL_VISIBLE
 GHashTable		*as_component_get_languages_table (AsComponent *cpt);
 
 void			as_component_set_bundles_array (AsComponent *cpt,
 							GPtrArray *bundles);
 
-AS_INTERNAL_VISIBLE
-GHashTable		*as_component_get_urls_table (AsComponent *cpt);
-
 gboolean		as_component_has_package (AsComponent *cpt);
 gboolean		as_component_has_install_candidate (AsComponent *cpt);
 
@@ -128,6 +105,32 @@ void			as_component_merge_with_mode (AsComponent *cpt,
 							AsComponent *source,
 							AsMergeKind merge_kind);
 
+AsContext		*as_component_get_context (AsComponent *cpt);
+void			as_component_set_context (AsComponent *cpt,
+						  AsContext *context);
+
+gboolean		as_component_load_from_xml (AsComponent *cpt,
+							AsContext *ctx,
+							xmlNode *node,
+							GError **error);
+xmlNode			*as_component_to_xml_node (AsComponent *cpt,
+							AsContext *ctx,
+							xmlNode *root);
+
+gboolean		as_component_load_from_yaml (AsComponent *cpt,
+						     AsContext *ctx,
+						     GNode *root,
+						     GError **error);
+void			as_component_emit_yaml (AsComponent *cpt,
+						AsContext *ctx,
+						yaml_emitter_t *emitter);
+
+void			as_component_to_variant (AsComponent *cpt,
+						 GVariantBuilder *builder);
+gboolean		as_component_set_from_variant (AsComponent *cpt,
+							GVariant *variant,
+							const gchar *locale);
+
 #pragma GCC visibility pop
 G_END_DECLS
 
