@@ -35,6 +35,9 @@
 #include "as-suggested.h"
 #include "as-category.h"
 #include "as-bundle.h"
+#include "as-content-rating.h"
+#include "as-launchable.h"
+#include "as-relation.h"
 
 G_BEGIN_DECLS
 
@@ -83,6 +86,7 @@ typedef enum  {
 	AS_COMPONENT_KIND_FIRMWARE,
 	AS_COMPONENT_KIND_DRIVER,
 	AS_COMPONENT_KIND_LOCALIZATION,
+	AS_COMPONENT_KIND_SERVICE,
 	/*< private >*/
 	AS_COMPONENT_KIND_LAST
 } AsComponentKind;
@@ -112,6 +116,22 @@ typedef enum  {
 const gchar		*as_merge_kind_to_string (AsMergeKind kind);
 AsMergeKind		as_merge_kind_from_string (const gchar *kind_str);
 
+/**
+ * AsComponentScope:
+ * @AS_COMPONENT_SCOPE_UNKNOWN:		Unknown scope
+ * @AS_COMPONENT_SCOPE_SYSTEM:		System scope
+ * @AS_COMPONENT_SCOPE_USER:		User scope
+ *
+ * Scope of the #AsComponent (system-wide or user-scope)
+ **/
+typedef enum {
+	AS_COMPONENT_SCOPE_UNKNOWN,
+	AS_COMPONENT_SCOPE_SYSTEM,
+	AS_COMPONENT_SCOPE_USER,
+	/*< private >*/
+	AS_COMPONENT_SCOPE_LAST
+} AsComponentScope;
+
 /**
  * AsValueFlags:
  * @AS_VALUE_FLAG_NONE:				No flags.
@@ -133,7 +153,7 @@ AsValueFlags		as_component_get_value_flags (AsComponent *cpt);
 void			as_component_set_value_flags (AsComponent *cpt,
 						      AsValueFlags flags);
 
-gchar			*as_component_get_active_locale (AsComponent *cpt);
+const gchar		*as_component_get_active_locale (AsComponent *cpt);
 void			as_component_set_active_locale (AsComponent *cpt,
 							const gchar *locale);
 
@@ -149,8 +169,6 @@ AsComponentKind		as_component_get_kind (AsComponent *cpt);
 void			as_component_set_kind (AsComponent *cpt,
 						AsComponentKind value);
 
-const gchar		*as_component_get_desktop_id (AsComponent *cpt);
-
 const gchar		*as_component_get_origin (AsComponent *cpt);
 void			as_component_set_origin (AsComponent *cpt,
 							const gchar *origin);
@@ -179,6 +197,12 @@ void			as_component_set_description (AsComponent *cpt,
 							const gchar *value,
 							const gchar *locale);
 
+GPtrArray		*as_component_get_launchables (AsComponent *cpt);
+AsLaunchable		*as_component_get_launchable (AsComponent *cpt,
+							AsLaunchableKind kind);
+void			as_component_add_launchable (AsComponent *cpt,
+							 AsLaunchable *launchable);
+
 const gchar		*as_component_get_metadata_license (AsComponent *cpt);
 void			as_component_set_metadata_license (AsComponent *cpt,
 							   const gchar *value);
@@ -295,6 +319,23 @@ gboolean		as_component_insert_custom_value (AsComponent *cpt,
 							  const gchar *key,
 							  const gchar *value);
 
+GPtrArray		*as_component_get_content_ratings (AsComponent *cpt);
+AsContentRating		*as_component_get_content_rating (AsComponent *cpt,
+							  const gchar *kind);
+void			as_component_add_content_rating (AsComponent *cpt,
+							 AsContentRating *content_rating);
+
+GPtrArray		*as_component_get_recommends (AsComponent *cpt);
+GPtrArray		*as_component_get_requires (AsComponent *cpt);
+void			as_component_add_relation (AsComponent *cpt,
+							AsRelation *relation);
+
+
+/* DEPRECATED */
+
+G_GNUC_DEPRECATED
+const gchar		*as_component_get_desktop_id (AsComponent *cpt);
+
 G_END_DECLS
 
 #endif /* __AS_COMPONENT_H */
