@@ -94,6 +94,8 @@ AsFormatKind		 as_format_kind_from_string (const gchar *kind_str);
  * @AS_FORMAT_VERSION_V0_8:	0.8
  * @AS_FORMAT_VERSION_V0_9:	0.9
  * @AS_FORMAT_VERSION_V0_10:	0.10
+ * @AS_FORMAT_VERSION_V0_11:	0.11
+ * @AS_FORMAT_VERSION_V0_12:	0.12
  *
  * Format version / API level of the AppStream metadata.
  **/
@@ -103,6 +105,8 @@ typedef enum {
 	AS_FORMAT_VERSION_V0_8,
 	AS_FORMAT_VERSION_V0_9,
 	AS_FORMAT_VERSION_V0_10,
+	AS_FORMAT_VERSION_V0_11,
+	AS_FORMAT_VERSION_V0_12,
 	/*< private >*/
 	AS_FORMAT_VERSION_LAST
 } AsFormatVersion;
@@ -110,7 +114,7 @@ typedef enum {
 const gchar		*as_format_version_to_string (AsFormatVersion version);
 AsFormatVersion		 as_format_version_from_string (const gchar *version_str);
 
-#define AS_CURRENT_FORMAT_VERSION AS_FORMAT_VERSION_V0_10
+#define AS_CURRENT_FORMAT_VERSION AS_FORMAT_VERSION_V0_12
 
 /**
  * AsMetadataError:
@@ -118,6 +122,7 @@ AsFormatVersion		 as_format_version_from_string (const gchar *version_str);
  * @AS_METADATA_ERROR_PARSE:			Unable to parse the metadata file.
  * @AS_METADATA_ERROR_FORMAT_UNEXPECTED:	Expected collection metadata but got metainfo metadata, or vice versa.
  * @AS_METADATA_ERROR_NO_COMPONENT:		We expected a component in the pool, but couldn't find one.
+ * @AS_METADATA_ERROR_VALUE_MISSING:		A essential value is missing in the source document.
  *
  * A metadata processing error.
  **/
@@ -126,6 +131,7 @@ typedef enum {
 	AS_METADATA_ERROR_PARSE,
 	AS_METADATA_ERROR_FORMAT_UNEXPECTED,
 	AS_METADATA_ERROR_NO_COMPONENT,
+	AS_METADATA_ERROR_VALUE_MISSING,
 	/*< private >*/
 	AS_METADATA_ERROR_LAST
 } AsMetadataError;
