@@ -288,8 +288,8 @@ as_pool_add_component_internal (AsPool *pool, AsComponent *cpt, gboolean pedanti
 		}
 
 		if (existing_cpt != NULL) {
-			if (as_component_get_origin_kind (cpt) != AS_ORIGIN_KIND_DESKTOP_ENTRY) {
-				/* discard tthis component if we have anything better data already in the pool,
+			if (as_component_get_origin_kind (existing_cpt) != AS_ORIGIN_KIND_DESKTOP_ENTRY) {
+				/* discard this component if we have better data already in the pool,
 				 * which is basically anything *but* data from a .desktop file */
 				g_debug ("Ignored .desktop metadata for '%s': Better data exists.", cdid);
 				return FALSE;
@@ -1155,7 +1155,7 @@ as_pool_get_components_by_id (AsPool *pool, const gchar *cid)
  * @kind: An #AsProvidesKind
  * @item: The value of the provided item.
  *
- * Find components in the AppStream data pool whcih provide a certain item.
+ * Find components in the AppStream data pool which provide a certain item.
  *
  * Returns: (transfer container) (element-type AsComponent): an array of #AsComponent objects which have been found.
  */
@@ -1268,6 +1268,62 @@ as_pool_get_components_by_categories (AsPool *pool, gchar **categories)
 	return results;
 }
 
+/**
+ * as_pool_get_components_by_launchable:
+ * @pool: An instance of #AsPool.
+ * @kind: An #AsLaunchableKind
+ * @id: The ID of the launchable.
+ *
+ * Find components in the AppStream data pool which provide a specific launchable.
+ * See #AsLaunchable for details on launchables, or refer to the AppStream specification.
+ *
+ * Returns: (transfer container) (element-type AsComponent): an array of #AsComponent objects which have been found.
+ *
+ * Since: 0.11.4
+ */
+GPtrArray*
+as_pool_get_components_by_launchable (AsPool *pool,
+					      AsLaunchableKind kind,
+					      const gchar *id)
+{
+	AsPoolPrivate *priv = GET_PRIVATE (pool);
+	GHashTableIter iter;
+	gpointer value;
+	GPtrArray *results;
+
+	/* sanity check */
+	g_return_val_if_fail (id != NULL, NULL);
+
+	results = g_ptr_array_new_with_free_func (g_object_unref);
+	g_hash_table_iter_init (&iter, priv->cpt_table);
+	while (g_hash_table_iter_next (&iter, NULL, &value)) {
+		GPtrArray *launchables = NULL;
+		guint i;
+		AsComponent *cpt = AS_COMPONENT (value);
+
+		launchables = as_component_get_launchables (cpt);
+		for (i = 0; i < launchables->len; i++) {
+			guint j;
+			GPtrArray *entries;
+			AsLaunchable *launch = AS_LAUNCHABLE (g_ptr_array_index (launchables, i));
+
+			if (kind != AS_LAUNCHABLE_KIND_UNKNOWN) {
+				/* check if the kind matches. an unknown kind matches all provides types */
+				if (as_launchable_get_kind (launch) != kind)
+					continue;
+			}
+
+			entries = as_launchable_get_entries (launch);
+			for (j = 0; j < entries->len; j++) {
+				if (g_strcmp0 ((const gchar*) g_ptr_array_index (entries, j), id) == 0)
+					g_ptr_array_add (results, g_object_ref (cpt));
+			}
+		}
+	}
+
+	return results;
+}
+
 /**
  * as_pool_build_search_terms:
  *
@@ -1758,15 +1814,17 @@ as_pool_add_metadata_location_internal (AsPool *pool, const gchar *directory, gb
 	gboolean dir_added = FALSE;
 	gchar *path;
 
-	if (!g_file_test (directory, G_FILE_TEST_IS_DIR))
+	if (!g_file_test (directory, G_FILE_TEST_IS_DIR)) {
+		g_debug ("Not adding metadata location '%s': Is no directory", directory);
 		return;
+	}
 
 	/* metadata locations */
 	path = g_build_filename (directory, "xml", NULL);
 	if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
 		g_ptr_array_add (priv->xml_dirs, path);
 		dir_added = TRUE;
-		g_debug ("Added %s to metadata search path.", path);
+		g_debug ("Added %s to XML metadata search path.", path);
 	} else {
 		g_free (path);
 	}
@@ -1775,7 +1833,7 @@ as_pool_add_metadata_location_internal (AsPool *pool, const gchar *directory, gb
 	if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
 		g_ptr_array_add (priv->xml_dirs, path);
 		dir_added = TRUE;
-		g_debug ("Added %s to metadata search path.", path);
+		g_debug ("Added %s to XML metadata search path.", path);
 	} else {
 		g_free (path);
 	}
@@ -1784,7 +1842,7 @@ as_pool_add_metadata_location_internal (AsPool *pool, const gchar *directory, gb
 	if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
 		g_ptr_array_add (priv->yaml_dirs, path);
 		dir_added = TRUE;
-		g_debug ("Added %s to metadata search path.", path);
+		g_debug ("Added %s to YAML metadata search path.", path);
 	} else {
 		g_free (path);
 	}
@@ -1793,6 +1851,7 @@ as_pool_add_metadata_location_internal (AsPool *pool, const gchar *directory, gb
 		/* we didn't find metadata-specific directories, so let's watch to root path for both YAML and XML */
 		g_ptr_array_add (priv->xml_dirs, g_strdup (directory));
 		g_ptr_array_add (priv->yaml_dirs, g_strdup (directory));
+		g_debug ("Added %s to all metadata search paths.", directory);
 	}
 
 	/* icons */
