@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2016 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2016-2017 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -22,11 +22,37 @@
 #define __AS_RELEASE_PRIVATE_H
 
 #include "as-release.h"
+#include "as-xml.h"
+#include "as-yaml.h"
 
 G_BEGIN_DECLS
 #pragma GCC visibility push(hidden)
 
-GHashTable	*as_release_get_description_table (AsRelease *release);
+AsContext		*as_release_get_context (AsRelease *release);
+void			as_release_set_context (AsRelease *release,
+						AsContext *context);
+
+gboolean		as_release_load_from_xml (AsRelease *release,
+						  AsContext *ctx,
+						  xmlNode *node,
+						  GError **error);
+void			as_release_to_xml_node (AsRelease *release,
+						AsContext *ctx,
+						xmlNode *root);
+
+gboolean		as_release_load_from_yaml (AsRelease *release,
+						   AsContext *ctx,
+						   GNode *node,
+						   GError **error);
+void			as_release_emit_yaml (AsRelease *release,
+						AsContext *ctx,
+						yaml_emitter_t *emitter);
+
+void			as_release_to_variant (AsRelease *release,
+						GVariantBuilder *builder);
+gboolean		as_release_set_from_variant (AsRelease *release,
+						     GVariant *variant,
+						     const gchar *locale);
 
 #pragma GCC visibility pop
 G_END_DECLS
