@@ -109,6 +109,8 @@ as_screenshot_kind_from_string (const gchar *kind)
 		return AS_SCREENSHOT_KIND_DEFAULT;
 	if (g_strcmp0 (kind, "extra") == 0)
 		return AS_SCREENSHOT_KIND_EXTRA;
+	if ((g_strcmp0 (kind, "") == 0) || (kind == NULL))
+		return AS_SCREENSHOT_KIND_EXTRA;
 	return AS_SCREENSHOT_KIND_UNKNOWN;
 }
 
@@ -396,6 +398,7 @@ as_screenshot_load_from_xml (AsScreenshot *screenshot, AsContext *ctx, xmlNode *
 	AsScreenshotPrivate *priv = GET_PRIVATE (screenshot);
 	xmlNode *iter;
 	g_autofree gchar *prop = NULL;
+	gboolean children_found = FALSE;
 
 	prop = (gchar*) xmlGetProp (node, (xmlChar*) "type");
 	if (g_strcmp0 (prop, "default") == 0)
@@ -409,6 +412,7 @@ as_screenshot_load_from_xml (AsScreenshot *screenshot, AsContext *ctx, xmlNode *
 		if (iter->type != XML_ELEMENT_NODE)
 			continue;
 		node_name = (const gchar*) iter->name;
+		children_found = TRUE;
 
 		if (g_strcmp0 (node_name, "image") == 0) {
 			g_autoptr(AsImage) image = as_image_new ();
@@ -428,6 +432,19 @@ as_screenshot_load_from_xml (AsScreenshot *screenshot, AsContext *ctx, xmlNode *
 		}
 	}
 
+	if (!children_found) {
+		/* we are likely dealing with a legacy screenshot node, which does not have <image/> children,
+		 * but instead contains the screenshot URL as text. This was briefly supported in an older AppStream
+		 * version for metainfo files, but it should no longer be used.
+		 * We support it here only for legacy compatibility. */
+		g_autoptr(AsImage) image = as_image_new ();
+
+		if (as_image_load_from_xml (image, ctx, node, NULL))
+			as_screenshot_add_image (screenshot, image);
+		else
+			return FALSE; /* this screenshot is invalid */
+	}
+
 	/* propagate context - we do this last so the image list for the selected locale is rebuilt properly */
 	as_screenshot_set_context (screenshot, ctx);
 
@@ -566,8 +583,10 @@ as_screenshot_emit_yaml (AsScreenshot *screenshot, AsContext *ctx, yaml_emitter_
  *
  * Serialize the current active state of this object to a GVariant
  * for use in the on-disk binary cache.
+ *
+ * Returns: %TRUE if a screenhot was added to the @builder
  */
-void
+gboolean
 as_screenshot_to_variant (AsScreenshot *screenshot, GVariantBuilder *builder)
 {
 	AsScreenshotPrivate *priv = GET_PRIVATE (screenshot);
@@ -575,6 +594,10 @@ as_screenshot_to_variant (AsScreenshot *screenshot, GVariantBuilder *builder)
 	GVariantBuilder images_b;
 	GVariantBuilder scr_b;
 
+	/* do not add screenshot without images to the cache */
+	if (priv->images->len == 0)
+		return FALSE;
+
 	g_variant_builder_init (&images_b, G_VARIANT_TYPE_ARRAY);
 	for (i = 0; i < priv->images->len; i++)
 		as_image_to_variant (AS_IMAGE (g_ptr_array_index (priv->images, i)), &images_b);
@@ -586,6 +609,7 @@ as_screenshot_to_variant (AsScreenshot *screenshot, GVariantBuilder *builder)
 
 	g_variant_builder_add_value (builder, g_variant_builder_end (&scr_b));
 
+	return TRUE;
 }
 
 /**
