@@ -51,6 +51,7 @@ typedef struct
 
 	AsComponent *current_cpt;
 	gchar *current_fname;
+	gboolean check_urls;
 } AsValidatorPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AsValidator, as_validator, G_TYPE_OBJECT)
@@ -85,6 +86,7 @@ as_validator_init (AsValidator *validator)
 
 	priv->current_fname = NULL;
 	priv->current_cpt = NULL;
+	priv->check_urls = FALSE;
 }
 
 /**
@@ -195,6 +197,97 @@ as_validator_clear_issues (AsValidator *validator)
 	g_hash_table_remove_all (priv->issues);
 }
 
+/**
+ * as_validator_can_check_urls:
+ *
+ * Check whether we can validate URLs using curl.
+ */
+static gboolean
+as_validator_can_check_urls (AsValidator *validator)
+{
+	return g_file_test ("/usr/bin/curl", G_FILE_TEST_EXISTS);
+}
+
+/**
+ * as_validator_web_url_exists:
+ *
+ * Check if an URL exists using curl.
+ */
+static gboolean
+as_validator_web_url_exists (AsValidator *validator, const gchar *url)
+{
+	AsValidatorPrivate *priv = GET_PRIVATE (validator);
+	/* we use absolute paths here to avoid someone injecting malicious curl/wget into our environment */
+	const gchar *curl_bin = "/usr/bin/curl";
+	gint exit_status = 0;
+
+	/* do nothing and assume the URL exists if we shouldn't check URLs */
+	if (!priv->check_urls)
+		return TRUE;
+
+	if (g_file_test (curl_bin, G_FILE_TEST_EXISTS)) {
+		/* Normally we would use the --head option of curl here to only fetch the server headers.
+		 * However, there is quite a bunch of unfriendly/misconfigured servers out there that simply
+		 * refuse to answer HEAD requests.
+		 * So, to be compatible with more stuff, we tell curl to attempt to fetch the first byte of the
+		 * document and report failure. We intentionally do not follow redirects. */
+		const gchar *argv[11];
+		argv[0] = curl_bin;
+		argv[1] = "--output";
+		argv[2] = "/dev/null";
+		argv[3] = "--silent";
+		argv[4] = "--fail";
+		argv[5] = "--max-time";
+		argv[6] = "20"; /* timeout of 20s, so this times out before a buildsystem (like Meson) times out after 30s */
+		argv[7] = "-r";
+		argv[8] = "0-0";
+		argv[9] = url;
+		argv[10] = NULL;
+		g_spawn_sync (NULL, /* wdir */
+				(gchar**) argv,
+				NULL, /* env */
+				G_SPAWN_STDOUT_TO_DEV_NULL | G_SPAWN_STDERR_TO_DEV_NULL,
+				NULL, /* setup function */
+				NULL, /* user data */
+				NULL, /* stdin */
+				NULL, /* stderr */
+				&exit_status,
+				NULL);
+		return exit_status == 0;
+	} else {
+		/* we can't validate this because we have no wget/curl - we should have emitted an error about this already, so
+		 * we just return TRUE here to not spam the user in misleading error messages */
+		return TRUE;
+	}
+}
+
+/**
+ * as_validator_get_check_urls:
+ * @validator: a #AsValidator instance.
+ *
+ * Returns: %TRUE in case we check if remote URLs exist.
+ */
+gboolean
+as_validator_get_check_urls (AsValidator *validator)
+{
+	AsValidatorPrivate *priv = GET_PRIVATE (validator);
+	return priv->check_urls;
+}
+
+/**
+ * as_validator_set_check_urls:
+ * @validator: a #AsValidator instance.
+ *
+ * Set this value to make the #AsValidator check whether remote URLs
+ * actually exist.
+ */
+void
+as_validator_set_check_urls (AsValidator *validator, gboolean value)
+{
+	AsValidatorPrivate *priv = GET_PRIVATE (validator);
+	priv->check_urls = value;
+}
+
 /**
  * as_validator_check_type_property:
  **/
@@ -459,10 +552,11 @@ as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsC
 {
 	guint i;
 	g_auto(GStrv) cid_parts = NULL;
+	gboolean hyphen_found = FALSE;
 	g_autofree gchar *cid = (gchar*) xmlNodeGetContent (idnode);
 
-	cid_parts = g_strsplit (cid, ".", 3);
-	if (g_strv_length (cid_parts) != 3) {
+	cid_parts = g_strsplit (cid, ".", -1);
+	if (g_strv_length (cid_parts) < 3) {
 		if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_DESKTOP_APP) {
 			/* since the ID and .desktop-file-id are tied together, we can't make this an error for desktop apps */
 			as_validator_add_issue (validator, idnode,
@@ -501,6 +595,33 @@ as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsC
 					AS_ISSUE_KIND_VALUE_WRONG,
 					"The component ID [%s] contains an invalid character: '%s'", cid, c);
 		}
+
+		if (!hyphen_found && cid[i] == '-') {
+			hyphen_found = TRUE;
+			as_validator_add_issue (validator, idnode,
+						AS_ISSUE_IMPORTANCE_INFO,
+						AS_ISSUE_KIND_VALUE_WRONG,
+						"The component ID [%s] contains a hyphen/minus. Using a hyphen is strongly discouraged, to keep interoperability with other tools such as D-Bus. "
+						"Ideally, replace any hyphens with an underscore ('_')", cid);
+		}
+	}
+
+	/* check if any segment starts with a number */
+	for (i = 0; cid_parts[i] != NULL; i++) {
+		if (g_ascii_isdigit (cid_parts[i][0])) {
+			as_validator_add_issue (validator, idnode,
+						AS_ISSUE_IMPORTANCE_INFO,
+						AS_ISSUE_KIND_VALUE_WRONG,
+						"The component ID [%s] contains a segment starting with a number. Starting a segment of the reverse-DNS ID with a number is strongly discouraged, "
+						"to keep interoperability with other tools such as D-Bus. "
+						"Ideally, prefix these sections with an underscore (%s → _%s)", cid, cid_parts[i], cid_parts[i]);
+			break;
+		}
+	}
+
+
+	/* a hyphen in the ID is bad news, because we can't use the ID on DBus and it also clashes with other naming schemes */
+	if (g_strstr_len (cid, -1, "-") != NULL) {
 	}
 
 	/* project-group specific constraints on the ID */
@@ -510,19 +631,19 @@ as_validator_validate_component_id (AsValidator *validator, xmlNode *idnode, AsC
 			as_validator_add_issue (validator, idnode,
 						AS_ISSUE_IMPORTANCE_ERROR,
 						AS_ISSUE_KIND_VALUE_WRONG,
-						"The component is part of the Freedesktop project, but its id does not start with fd.o's reverse-DNS name (\"org.freedesktop\").");
+						"The component is part of the Freedesktop project, but its ID does not start with fd.o's reverse-DNS name (\"org.freedesktop\").");
 	} else if (g_strcmp0 (as_component_get_project_group (cpt), "KDE") == 0) {
 		if (!g_str_has_prefix (cid, "org.kde."))
 			as_validator_add_issue (validator, idnode,
 						AS_ISSUE_IMPORTANCE_ERROR,
 						AS_ISSUE_KIND_VALUE_WRONG,
-						"The component is part of the KDE project, but its id does not start with KDEs reverse-DNS name (\"org.kde\").");
+						"The component is part of the KDE project, but its ID does not start with KDEs reverse-DNS name (\"org.kde\").");
 	} else if (g_strcmp0 (as_component_get_project_group (cpt), "GNOME") == 0) {
 		if (!g_str_has_prefix (cid, "org.gnome."))
 			as_validator_add_issue (validator, idnode,
 						AS_ISSUE_IMPORTANCE_PEDANTIC,
 						AS_ISSUE_KIND_VALUE_WRONG,
-						"The component is part of the GNOME project, but its id does not start with GNOMEs reverse-DNS name (\"org.gnome\").");
+						"The component is part of the GNOME project, but its ID does not start with GNOMEs reverse-DNS name (\"org.gnome\").");
 	}
 }
 
@@ -564,6 +685,76 @@ as_validator_validate_project_license (AsValidator *validator, xmlNode *license_
 	}
 }
 
+/**
+ * as_validator_validate_metadata_license:
+ */
+static void
+as_validator_validate_metadata_license (AsValidator *validator, xmlNode *license_node)
+{
+	gboolean requires_all_tokens = TRUE;
+	guint license_bad_cnt = 0;
+	guint license_good_cnt = 0;
+	g_auto(GStrv) tokens = NULL;
+	g_autofree gchar *license_expression = (gchar*) xmlNodeGetContent (license_node);
+
+	tokens = as_spdx_license_tokenize (license_expression);
+	if (tokens == NULL) {
+		as_validator_add_issue (validator, license_node,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_WRONG,
+					"SPDX license expression '%s' could not be parsed.",
+					license_expression);
+		return;
+	}
+
+	/* this is too complicated to process */
+	for (guint i = 0; tokens[i] != NULL; i++) {
+		if (g_strcmp0 (tokens[i], "(") == 0 ||
+		    g_strcmp0 (tokens[i], ")") == 0) {
+			as_validator_add_issue (validator, license_node,
+						AS_ISSUE_IMPORTANCE_WARNING,
+						AS_ISSUE_KIND_VALUE_WRONG,
+						"The metadata itself seems to be licensed under a complex collection of licenses. Please license the data under a simple permissive license, like FSFAP, CC-0-1.0 or MIT "
+						"to allow distributors to include it in mixed data collections without the risk of license violations due to mutually incompatible licenses.");
+			return;
+		}
+	}
+
+	/* this is a simple expression parser and can be easily tricked */
+	for (guint i = 0; tokens[i] != NULL; i++) {
+		if (g_strcmp0 (tokens[i], "+") == 0)
+			continue;
+		if (g_strcmp0 (tokens[i], "|") == 0) {
+			requires_all_tokens = FALSE;
+			continue;
+		}
+		if (g_strcmp0 (tokens[i], "&") == 0) {
+			requires_all_tokens = TRUE;
+			continue;
+		}
+		if (as_license_is_metadata_license (tokens[i])) {
+			license_good_cnt++;
+		} else {
+			license_bad_cnt++;
+		}
+	}
+
+	/* any valid token makes this valid */
+	if (!requires_all_tokens && license_good_cnt > 0)
+		return;
+
+	/* all tokens are required to be valid */
+	if (requires_all_tokens && license_bad_cnt == 0)
+		return;
+
+	/* looks like the license was bad */
+	as_validator_add_issue (validator, license_node,
+				AS_ISSUE_IMPORTANCE_WARNING,
+				AS_ISSUE_KIND_VALUE_WRONG,
+				"The metadata itself does not seem to be licensed under a permissive license. Please license the data under a permissive license, like FSFAP, CC-0-1.0 or MIT "
+				"to allow distributors to include it in mixed data collections without the risk of license violations due to mutually incompatible licenses.");
+}
+
 /**
  * as_validator_validate_update_contact:
  */
@@ -585,6 +776,168 @@ as_validator_validate_update_contact (AsValidator *validator, xmlNode *uc_node)
 	}
 }
 
+/**
+ * as_validator_check_screenshots:
+ *
+ * Validate a "screenshots" tag.
+ **/
+static void
+as_validator_check_screenshots (AsValidator *validator, xmlNode *node, AsComponent *cpt)
+{
+	xmlNode *iter;
+	as_validator_check_children_quick (validator, node, "screenshot", cpt);
+
+	for (iter = node->children; iter != NULL; iter = iter->next) {
+		xmlNode *iter2;
+		gboolean image_found = FALSE;
+		gboolean caption_found = FALSE;
+
+		if (iter->type != XML_ELEMENT_NODE)
+			continue;
+
+		for (iter2 = iter->children; iter2 != NULL; iter2 = iter2->next) {
+			const gchar *node_name = (const gchar*) iter2->name;
+
+			if (iter2->type != XML_ELEMENT_NODE)
+				continue;
+
+			if (g_strcmp0 (node_name, "image") == 0) {
+				g_autofree gchar *image_url = (gchar*) xmlNodeGetContent (iter2);
+
+				image_found = TRUE;
+
+				if (!as_validator_web_url_exists (validator, image_url)) {
+					as_validator_add_issue (validator, iter2,
+							AS_ISSUE_IMPORTANCE_WARNING,
+							AS_ISSUE_KIND_REMOTE_ERROR,
+							"Unable to reach screenshot image on remote location '%s' - does the image exist?",
+							image_url);
+				}
+			} else if (g_strcmp0 (node_name, "caption") == 0) {
+				caption_found = TRUE;
+			} else {
+				as_validator_add_issue (validator, iter2,
+							AS_ISSUE_IMPORTANCE_WARNING,
+							AS_ISSUE_KIND_TAG_UNKNOWN,
+							"Found tag '%s' in a screenshot. Only <caption/> and <image/> tags are allowed.",
+							(const gchar*) iter2->name);
+			}
+		}
+
+		if (g_strcmp0 ((const gchar*) iter->name, "screenshot") != 0) {
+			as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_WARNING,
+							AS_ISSUE_KIND_TAG_UNKNOWN,
+							"Found tag '%s' in a screenshots group. Only <screenshot/> tags are allowed.",
+							(const gchar*) iter->name);
+		}
+
+		if (!image_found) {
+			as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_TAG_MISSING,
+						"The screenshot does not contain any images.");
+		}
+
+		if (!caption_found) {
+			as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_PEDANTIC,
+						AS_ISSUE_KIND_TAG_MISSING,
+						"The screenshot does not have a caption text.");
+		}
+	}
+}
+
+/**
+ * as_validator_check_requires_recommends:
+ **/
+static void
+as_validator_check_requires_recommends (AsValidator *validator, xmlNode *node, AsComponent *cpt, AsRelationKind kind)
+{
+	xmlNode *iter;
+
+	for (iter = node->children; iter != NULL; iter = iter->next) {
+		const gchar *node_name;
+		g_autofree gchar *content = NULL;
+		g_autofree gchar *version = NULL;
+		gboolean can_have_version;
+		AsRelationItemKind item_kind;
+
+		/* discard spaces */
+		if (iter->type != XML_ELEMENT_NODE)
+			continue;
+		node_name = (const gchar*) iter->name;
+		content = as_xml_get_node_value (iter);
+
+		item_kind = as_relation_item_kind_from_string (node_name);
+		if (item_kind == AS_RELATION_ITEM_KIND_UNKNOWN) {
+			as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_WARNING,
+						AS_ISSUE_KIND_TAG_UNKNOWN,
+						"Found tag '%s' in a requires/recommends group. A relation of this type is unknown.",
+						node_name);
+			continue;
+		}
+
+		if (g_strcmp0 (content, "") == 0) {
+			as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_VALUE_MISSING,
+						"Missing value for requires/recommends item.");
+			continue;
+		}
+
+		switch (item_kind) {
+		case AS_RELATION_ITEM_KIND_MEMORY:
+		case AS_RELATION_ITEM_KIND_MODALIAS:
+			can_have_version = FALSE;
+			break;
+		default:
+			can_have_version = TRUE;
+		}
+
+		version = (gchar*) xmlGetProp (iter, (xmlChar*) "version");
+		if (version != NULL) {
+			AsRelationCompare compare;
+			g_autofree gchar *compare_str = (gchar*) xmlGetProp (iter, (xmlChar*) "compare");
+
+			if (!can_have_version) {
+				as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_WARNING,
+						AS_ISSUE_KIND_PROPERTY_INVALID,
+						"Found version property on required/recommended item of type '%s'. Items of this type should not have a version.",
+						as_relation_item_kind_to_string (item_kind));
+				continue;
+			}
+
+			if (compare_str == NULL) {
+				as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_INFO,
+						AS_ISSUE_KIND_PROPERTY_MISSING,
+						"Found version property on required/recommended item, but not 'compare' property. It is recommended to explicitly define a comparison operation.");
+				continue;
+			}
+
+			compare = as_relation_compare_from_string (compare_str);
+			if (compare == AS_RELATION_COMPARE_UNKNOWN) {
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_WRONG,
+							"Invalid version comparison operation '%s' on item.",
+							compare_str);
+			}
+		}
+
+		if ((kind == AS_RELATION_KIND_REQUIRES) && (item_kind == AS_RELATION_ITEM_KIND_MEMORY)) {
+			as_validator_add_issue (validator, iter,
+						AS_ISSUE_IMPORTANCE_INFO,
+						AS_ISSUE_KIND_UNUSUAL,
+						"Found a memory size dependency in a 'requires' tag. This means users will not be able to even install the component without having enough RAM." " "
+					        "This is usually not intended and you want to use 'memory' in the 'recommends' tag instead.");
+		}
+	}
+}
+
 /**
  * as_validator_validate_component_node:
  **/
@@ -633,14 +986,23 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 					"The component has a 'merge' method defined. This is not allowed in metainfo files.");
 	}
 
+	/* the component must have an id */
+	if (as_str_empty (as_component_get_id (cpt))) {
+		/* we don't have an id */
+		as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_VALUE_MISSING,
+					"The component is missing an ID (<id/> tag).");
+	}
+
 	/* the component must have a name */
 	if (as_str_empty (as_component_get_name (cpt))) {
-		/* we don't have a summary */
+		/* we don't have a name */
 		as_validator_add_issue (validator, NULL,
 					AS_ISSUE_IMPORTANCE_ERROR,
 					AS_ISSUE_KIND_VALUE_MISSING,
 					"The component is missing a name (<name/> tag).");
-		}
+	}
 
 	/* the component must have a summary */
 	if (as_str_empty (as_component_get_summary (cpt))) {
@@ -680,15 +1042,8 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 
 			/* the license must allow easy mixing of metadata in metainfo files */
-			if (mode == AS_FORMAT_STYLE_METAINFO) {
-				if (!as_license_is_metadata_license (node_content)) {
-					as_validator_add_issue (validator, iter,
-								AS_ISSUE_IMPORTANCE_WARNING,
-								AS_ISSUE_KIND_VALUE_WRONG,
-								"The metadata itself does not seem to be licensed under a permissive license. Please license the data under a permissive license, like FSFAP, CC-0-1.0 or MIT "
-								"to allow distributors to include it in mixed data collections without the risk of license violations due to mutually incompatible licenses.");
-				}
-			}
+			if (mode == AS_FORMAT_STYLE_METAINFO)
+				as_validator_validate_metadata_license (validator, iter);
 		} else if (g_strcmp0 (node_name, "pkgname") == 0) {
 			if (g_hash_table_contains (found_tags, node_name)) {
 				as_validator_add_issue (validator, iter,
@@ -749,19 +1104,28 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 			}
 
 			if (g_strcmp0 (prop, "remote") == 0) {
-				if (!as_validate_is_url (node_content))
+				if (!as_validate_is_url (node_content)) {
 					as_validator_add_issue (validator, iter,
 								AS_ISSUE_IMPORTANCE_ERROR,
 								AS_ISSUE_KIND_VALUE_WRONG,
 								"Icons of type 'remote' must contain an URL to the referenced icon.");
+				} else {
+					if (!as_validator_web_url_exists (validator, node_content)) {
+						as_validator_add_issue (validator, iter,
+									AS_ISSUE_IMPORTANCE_WARNING,
+									AS_ISSUE_KIND_REMOTE_ERROR,
+									"Unable to reach remote icon at '%s' - does it exist?",
+									node_content);
+					}
+				}
 			}
 
 			if (mode == AS_FORMAT_STYLE_METAINFO) {
-				if ((prop != NULL) && (g_strcmp0 (prop, "stock") != 0)) {
+				if ((prop != NULL) && (g_strcmp0 (prop, "stock") != 0) && (g_strcmp0 (prop, "remote") != 0)) {
 					as_validator_add_issue (validator, iter,
 								AS_ISSUE_IMPORTANCE_ERROR,
 								AS_ISSUE_KIND_VALUE_WRONG,
-								"Metainfo files may only contain 'stock' icons, icons of kind '%s' are not allowed.", prop);
+								"Metainfo files may only contain 'stock' or 'remote' icons, icons of kind '%s' are not allowed.", prop);
 				}
 			}
 			g_free (prop);
@@ -776,6 +1140,14 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 							prop);
 			}
 			g_free (prop);
+
+			if (!as_validator_web_url_exists (validator, node_content)) {
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_WARNING,
+							AS_ISSUE_KIND_REMOTE_ERROR,
+							"Unable to reach remote location '%s' - does it exist?",
+							node_content);
+			}
 		} else if (g_strcmp0 (node_name, "categories") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			as_validator_check_children_quick (validator, iter, "category", cpt);
@@ -788,7 +1160,7 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 		} else if (g_strcmp0 (node_name, "provides") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 		} else if (g_strcmp0 (node_name, "screenshots") == 0) {
-			as_validator_check_children_quick (validator, iter, "screenshot", cpt);
+			as_validator_check_screenshots (validator, iter, cpt);
 		} else if (g_strcmp0 (node_name, "project_license") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			as_validator_validate_project_license (validator, iter);
@@ -827,6 +1199,14 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 							"Unknown type '%s' for <translation/> tag.", prop);
 			}
 		} else if (g_strcmp0 (node_name, "launchable") == 0) {
+			g_autofree gchar *prop = NULL;
+			prop = as_validator_check_type_property (validator, cpt, iter);
+			if (as_launchable_kind_from_string (prop) == AS_LAUNCHABLE_KIND_UNKNOWN) {
+				as_validator_add_issue (validator, iter,
+							AS_ISSUE_IMPORTANCE_ERROR,
+							AS_ISSUE_KIND_VALUE_WRONG,
+							"Unknown type '%s' for <launchable/> tag.", prop);
+			}
 		} else if (g_strcmp0 (node_name, "extends") == 0) {
 		} else if (g_strcmp0 (node_name, "bundle") == 0) {
 			g_autofree gchar *prop = NULL;
@@ -851,6 +1231,10 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 			as_validator_check_children_quick (validator, iter, "id", cpt);
 		} else if (g_strcmp0 (node_name, "content_rating") == 0) {
 			as_validator_check_children_quick (validator, iter, "content_attribute", cpt);
+		} else if (g_strcmp0 (node_name, "requires") == 0) {
+			as_validator_check_requires_recommends (validator, iter, cpt, AS_RELATION_KIND_REQUIRES);
+		} else if (g_strcmp0 (node_name, "recommends") == 0) {
+			as_validator_check_requires_recommends (validator, iter, cpt, AS_RELATION_KIND_RECOMMENDS);
 		} else if (g_strcmp0 (node_name, "custom") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			as_validator_check_children_quick (validator, iter, "value", cpt);
@@ -929,6 +1313,31 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 					"Type 'console-application' component, but no information about binaries in $PATH was provided via a provides/binary tag.");
 	}
 
+	/* validate webapp specific stuff */
+	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_WEB_APP) {
+		AsLaunchable *launch = as_component_get_launchable (cpt, AS_LAUNCHABLE_KIND_URL);
+		if (launch == NULL || as_launchable_get_entries (launch)->len == 0) {
+			as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_TAG_MISSING,
+					"This 'web-application' component is missing a 'launchable' tag of type 'url'.");
+		}
+
+		if (as_component_get_icons (cpt)->len <= 0) {
+			as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_TAG_MISSING,
+					"This 'web-application' component is missing a 'icon' tag to specify a valid icon.");
+		}
+
+		if (as_component_get_categories (cpt)->len <= 0) {
+			as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_TAG_MISSING,
+					"This 'web-application' component is missing categorizations. A 'categories' block is likely missing.");
+		}
+	}
+
 	/* validate font specific stuff */
 	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_FONT) {
 		if (as_component_get_provided_for_kind (cpt, AS_PROVIDED_KIND_FONT) == NULL)
@@ -1042,13 +1451,6 @@ as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xm
 							"The screenshot caption '%s' is too long (should be <= 80 characters)",
 							scr_caption);
 			}
-
-			if (as_screenshot_get_images (scr)->len <= 0) {
-				as_validator_add_issue (validator, NULL,
-							AS_ISSUE_IMPORTANCE_ERROR,
-							AS_ISSUE_KIND_TAG_MISSING,
-							"The component contains a screenshot without any images.");
-			}
 		}
 	}
 
@@ -1164,12 +1566,24 @@ as_validator_open_xml_document (AsValidator *validator, const gchar *xmldata)
 gboolean
 as_validator_validate_data (AsValidator *validator, const gchar *metadata)
 {
+	AsValidatorPrivate *priv = GET_PRIVATE (validator);
 	gboolean ret;
 	xmlNode* root;
 	xmlDoc *doc;
 	g_autoptr(AsContext) ctx = NULL;
 	AsComponent *cpt;
 
+	/* if we validate URLs, check if curl or wget are installed */
+	if (priv->check_urls) {
+		/* cheap way to notify the user if we can't validate URLs */
+		if (!as_validator_can_check_urls (validator)) {
+			as_validator_add_issue (validator, NULL,
+						AS_ISSUE_IMPORTANCE_INFO,
+						AS_ISSUE_KIND_UNKNOWN,
+						"Unable to find the curl binary. remote URLs can not be checked for validity!");
+		}
+	}
+
 	/* load the XML data */
 	ctx = as_context_new ();
 	as_context_set_locale (ctx, "C");
@@ -1284,7 +1698,7 @@ as_validator_analyze_component_metainfo_relation_cb (const gchar *fname, AsCompo
 		cid_base = g_strdup (as_component_get_id (cpt));
 	}
 	if (!as_matches_metainfo (fname, cid_base)) {
-		/* the name-without-type didn't match - check for the full id in the component name */
+		/* the name-without-type didn't match - check for the full ID in the component name */
 		if (!as_matches_metainfo (fname, as_component_get_id (cpt))) {
 			as_validator_add_issue (data->validator, NULL,
 					AS_ISSUE_IMPORTANCE_WARNING,
@@ -1364,6 +1778,7 @@ as_validator_analyze_component_metainfo_relation_cb (const gchar *fname, AsCompo
 gboolean
 as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 {
+	AsValidatorPrivate *priv = GET_PRIVATE (validator);
 	g_autofree gchar *metainfo_dir = NULL;
 	g_autofree gchar *legacy_metainfo_dir = NULL;
 	g_autofree gchar *apps_dir = NULL;
@@ -1402,6 +1817,17 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 					"No XDG applications directory found.");
 	}
 
+	/* if we validate URLs, check if curl or wget are installed */
+	if (priv->check_urls) {
+		/* cheap way to notify the user if we can't validate URLs */
+		if (!as_validator_can_check_urls (validator)) {
+			as_validator_add_issue (validator, NULL,
+						AS_ISSUE_IMPORTANCE_INFO,
+						AS_ISSUE_KIND_UNKNOWN,
+						"Unable to find the curl binary. remote URLs can not be checked for validity!");
+		}
+	}
+
 	/* holds a filename -> component mapping */
 	validated_cpts = g_hash_table_new_full (g_str_hash,
 						g_str_equal,
