@@ -356,7 +356,7 @@ test_pool_read ()
 		if (g_strcmp0 (cat_id, "communication") == 0)
 			g_assert_cmpint (cpt_count, ==, 3);
 		else if (g_strcmp0 (cat_id, "utilities") == 0)
-			g_assert_cmpint (cpt_count, ==, 2);
+			g_assert_cmpint (cpt_count, ==, 3);
 		else if (g_strcmp0 (cat_id, "audio-video") == 0)
 			g_assert_cmpint (cpt_count, ==, 0);
 		else if (g_strcmp0 (cat_id, "developer-tools") == 0)
@@ -364,7 +364,7 @@ test_pool_read ()
 		else if (g_strcmp0 (cat_id, "education") == 0)
 			g_assert_cmpint (cpt_count, ==, 4);
 		else if (g_strcmp0 (cat_id, "games") == 0)
-			g_assert_cmpint (cpt_count, ==, 5);
+			g_assert_cmpint (cpt_count, ==, 4);
 		else if (g_strcmp0 (cat_id, "graphics") == 0)
 			g_assert_cmpint (cpt_count, ==, 1);
 		else if (g_strcmp0 (cat_id, "office") == 0)
@@ -383,6 +383,17 @@ test_pool_read ()
 			g_assert_cmpstr (as_component_get_id (cpt), ==, "org.inkscape.Inkscape");
 		}
 	}
+
+	/* test fetching components by launchable */
+	result = as_pool_get_components_by_launchable (dpool, AS_LAUNCHABLE_KIND_DESKTOP_ID, "linuxdcpp.desktop");
+	g_assert_cmpint (result->len, ==, 1);
+	g_ptr_array_unref (result);
+
+	result = as_pool_get_components_by_launchable (dpool, AS_LAUNCHABLE_KIND_DESKTOP_ID, "inkscape.desktop");
+	g_assert_cmpint (result->len, ==, 1);
+	cpt = AS_COMPONENT (g_ptr_array_index (result, 0));
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.inkscape.Inkscape");
+	g_ptr_array_unref (result);
 }
 
 /**
