@@ -22,7 +22,7 @@
 #include <glib/gprintf.h>
 
 #include "appstream.h"
-#include "as-yamldata.h"
+#include "as-metadata.h"
 #include "as-test-utils.h"
 
 static gchar *datadir = NULL;
@@ -40,7 +40,7 @@ test_basic (void)
 	GFile *file;
 	GPtrArray *cpts;
 	guint i;
-	AsComponent *cpt_tomatoes;
+	AsComponent *cpt_tomatoes = NULL;
 	GError *error = NULL;
 
 	mdata = as_metadata_new ();
@@ -67,7 +67,7 @@ test_basic (void)
 	}
 
 	/* just check one of the components... */
-	g_assert (cpt_tomatoes != NULL);
+	g_assert_nonnull (cpt_tomatoes);
 	g_assert_cmpstr (as_component_get_summary (cpt_tomatoes), ==, "How many tomatoes can you smash in ten short minutes?");
 	g_assert_cmpstr (as_component_get_pkgnames (cpt_tomatoes)[0], ==, "tomatoes");
 }
@@ -118,16 +118,14 @@ static gchar*
 as_yaml_test_serialize (AsComponent *cpt)
 {
 	gchar *data;
-	g_autoptr(GPtrArray) cpts = NULL;
-	g_autoptr(AsYAMLData) ydt = NULL;
+	g_autoptr(AsMetadata) metad = NULL;
 	GError *error = NULL;
 
-	ydt = as_yamldata_new ();
-	as_yamldata_set_check_valid (ydt, FALSE);
+	metad = as_metadata_new ();
+	as_metadata_add_component (metad, cpt);
+	as_metadata_set_write_header (metad, TRUE);
 
-	cpts = g_ptr_array_new ();
-	g_ptr_array_add (cpts, cpt);
-	data = as_yamldata_serialize_to_collection (ydt, cpts, TRUE, FALSE, &error);
+	data = as_metadata_components_to_collection (metad, AS_FORMAT_KIND_YAML, &error);
 	g_assert_no_error (error);
 
 	return data;
@@ -142,8 +140,7 @@ static void
 test_yamlwrite_general (void)
 {
 	guint i;
-	g_autoptr(AsYAMLData) ydata = NULL;
-	g_autoptr(GPtrArray) cpts = NULL;
+	g_autoptr(AsMetadata) metad = NULL;
 	g_autoptr(AsScreenshot) scr = NULL;
 	g_autoptr(AsRelease) rel1 = NULL;
 	g_autoptr(AsRelease) rel2 = NULL;
@@ -156,7 +153,7 @@ test_yamlwrite_general (void)
 
 	const gchar *expected_yaml = "---\n"
 				"File: DEP-11\n"
-				"Version: '0.10'\n"
+				"Version: '0.12'\n"
 				"---\n"
 				"Type: firmware\n"
 				"ID: org.example.test.firmware\n"
@@ -213,6 +210,7 @@ test_yamlwrite_general (void)
 				"  percentage: 100\n"
 				"Releases:\n"
 				"- version: '1.0'\n"
+				"  type: development\n"
 				"  unix-timestamp: 1460463132\n"
 				"  description:\n"
 				"    de_DE: >-\n"
@@ -224,6 +222,7 @@ test_yamlwrite_general (void)
 				"\n"
 				"      <p>Second paragraph.</p>\n"
 				"- version: '1.2'\n"
+				"  type: stable\n"
 				"  unix-timestamp: 1462288512\n"
 				"  urgency: medium\n"
 				"  description:\n"
@@ -236,8 +235,7 @@ test_yamlwrite_general (void)
 				"Name:\n"
 				"  C: ReplaceThis!\n";
 
-	ydata = as_yamldata_new ();
-	cpts = g_ptr_array_new_with_free_func (g_object_unref);
+	metad = as_metadata_new ();
 
 	/* firmware component */
 	cpt = as_component_new ();
@@ -250,7 +248,8 @@ test_yamlwrite_general (void)
 	as_component_add_extends (cpt, "org.example.alpha");
 	as_component_add_extends (cpt, "org.example.beta");
 	as_component_add_url (cpt, AS_URL_KIND_HOMEPAGE, "https://example.com");
-	g_ptr_array_add (cpts, cpt);
+	as_metadata_add_component (metad, cpt);
+	g_object_unref (cpt);
 
 	/* component with icons, screenshots and release descriptions */
 	cpt = as_component_new ();
@@ -285,6 +284,7 @@ test_yamlwrite_general (void)
 
 	rel1 = as_release_new ();
 	as_release_set_version (rel1, "1.0");
+	as_release_set_kind (rel1, AS_RELEASE_KIND_DEVELOPMENT);
 	as_release_set_timestamp (rel1, 1460463132);
 	as_release_set_description (rel1, "<p>Awesome initial release.</p>\n<p>Second paragraph.</p>", "C");
 	as_release_set_description (rel1, "<p>Großartige erste Veröffentlichung.</p>\n<p>Zweite zeile.</p>", "de_DE");
@@ -303,7 +303,8 @@ test_yamlwrite_general (void)
 	as_bundle_set_id (bdl, "foobar");
 	as_component_add_bundle (cpt, bdl);
 
-	g_ptr_array_add (cpts, cpt);
+	as_metadata_add_component (metad, cpt);
+	g_object_unref (cpt);
 
 	/* merge component */
 	cpt = as_component_new ();
@@ -311,10 +312,11 @@ test_yamlwrite_general (void)
 	as_component_set_merge_kind (cpt, AS_MERGE_KIND_REPLACE);
 	as_component_set_id (cpt, "org.example.ATargetComponent");
 	as_component_set_name (cpt, "ReplaceThis!", "C");
-	g_ptr_array_add (cpts, cpt);
+	as_metadata_add_component (metad, cpt);
+	g_object_unref (cpt);
 
 	/* serialize and validate */
-	resdata = as_yamldata_serialize_to_collection (ydata, cpts, TRUE, FALSE, &error);
+	resdata = as_metadata_components_to_collection (metad, AS_FORMAT_KIND_YAML, &error);
 	g_assert_no_error (error);
 
 	g_assert (as_test_compare_lines (resdata, expected_yaml));
@@ -329,25 +331,30 @@ static AsComponent*
 as_yaml_test_read_data (const gchar *data, GError **error)
 {
 	AsComponent *cpt;
-	g_autoptr(GPtrArray) cpts = NULL;
-	g_autoptr(AsYAMLData) ydt = NULL;
+	g_autoptr(AsMetadata) metad = NULL;
 
-	ydt = as_yamldata_new ();
+	metad = as_metadata_new ();
+	as_metadata_set_format_style (metad, AS_FORMAT_STYLE_COLLECTION);
 
 	if (error == NULL) {
 		g_autoptr(GError) local_error = NULL;
-
-		cpts = as_yamldata_parse_collection_data (ydt, data, &local_error);
+		as_metadata_parse (metad, data, AS_FORMAT_KIND_YAML, &local_error);
 		g_assert_no_error (local_error);
-		g_assert_nonnull (cpts);
-	} else {
-		cpts = as_yamldata_parse_collection_data (ydt, data, error);
-		if (cpts == NULL)
-			return NULL;
-	}
 
-	cpt = AS_COMPONENT (g_ptr_array_index (cpts, 0));
-	return g_object_ref (cpt);
+		g_assert_cmpint (as_metadata_get_components (metad)->len, >, 0);
+		cpt = AS_COMPONENT (g_ptr_array_index (as_metadata_get_components (metad), 0));
+
+		return g_object_ref (cpt);
+	} else {
+		as_metadata_parse (metad, data, AS_FORMAT_KIND_YAML, error);
+
+		if (as_metadata_get_components (metad)->len > 0) {
+			cpt = AS_COMPONENT (g_ptr_array_index (as_metadata_get_components (metad), 0));
+			return g_object_ref (cpt);
+		} else {
+			return NULL;
+		}
+	}
 }
 
 /**
@@ -362,7 +369,6 @@ test_yaml_read_icons (void)
 	GPtrArray *icons;
 	g_autoptr(AsComponent) cpt = NULL;
 
-	g_autoptr(AsYAMLData) ydt = NULL;
 	const gchar *yamldata_icons_legacy = "---\n"
 					"ID: org.example.Test\n"
 					"Icon:\n"
@@ -375,6 +381,10 @@ test_yaml_read_icons (void)
 					"    - width: 64\n"
 					"      height: 64\n"
 					"      name: test_test.png\n"
+					"    - width: 64\n"
+					"      height: 64\n"
+					"      name: test_test.png\n"
+					"      scale: 2\n"
 					"    - width: 128\n"
 					"      height: 128\n"
 					"      name: test_test.png\n"
@@ -387,8 +397,6 @@ test_yaml_read_icons (void)
 					"      height: 64\n"
 					"      name: single_test.png\n";
 
-	ydt = as_yamldata_new ();
-
 	/* check the legacy icons */
 	cpt = as_yaml_test_read_data (yamldata_icons_legacy, NULL);
 	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.Test");
@@ -410,7 +418,7 @@ test_yaml_read_icons (void)
 	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.Test");
 
 	icons = as_component_get_icons (cpt);
-	g_assert_cmpint (icons->len, ==, 3);
+	g_assert_cmpint (icons->len, ==, 4);
 	for (i = 0; i < icons->len; i++) {
 		AsIcon *icon = AS_ICON (g_ptr_array_index (icons, i));
 
@@ -516,7 +524,7 @@ test_yaml_write_suggests (void)
 	g_autofree gchar *res = NULL;
 	const gchar *expected_sug_yaml = "---\n"
 					 "File: DEP-11\n"
-					 "Version: '0.10'\n"
+					 "Version: '0.12'\n"
 					 "---\n"
 					 "Type: generic\n"
 					 "ID: org.example.SuggestsTest\n"
@@ -597,7 +605,7 @@ test_yaml_read_suggests (void)
 
 static const gchar *yamldata_custom_field = "---\n"
 					 "File: DEP-11\n"
-					 "Version: '0.10'\n"
+					 "Version: '0.12'\n"
 					 "---\n"
 					 "Type: generic\n"
 					 "ID: org.example.CustomTest\n"
@@ -647,6 +655,248 @@ test_yaml_read_custom (void)
 	g_assert_cmpstr (as_component_get_custom_value (cpt, "Oh::Snap::Punctuation!"), ==, "Awesome!");
 }
 
+static const gchar *yamldata_content_rating_field = "---\n"
+						"File: DEP-11\n"
+						"Version: '0.12'\n"
+						"---\n"
+						"Type: generic\n"
+						"ID: org.example.ContentRatingTest\n"
+						"ContentRating:\n"
+						"  oars-1.0:\n"
+						"    drugs-alcohol: moderate\n"
+						"    language-humor: mild\n";
+
+/**
+ * test_yaml_write_content_rating:
+ *
+ * Test writing the ContentRating field.
+ */
+static void
+test_yaml_write_content_rating (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autoptr(AsContentRating) rating = NULL;
+	g_autofree gchar *res = NULL;
+
+	cpt = as_component_new ();
+	as_component_set_kind (cpt, AS_COMPONENT_KIND_GENERIC);
+	as_component_set_id (cpt, "org.example.ContentRatingTest");
+
+	rating = as_content_rating_new ();
+	as_content_rating_set_kind (rating, "oars-1.0");
+
+	as_content_rating_set_value (rating, "drugs-alcohol", AS_CONTENT_RATING_VALUE_MODERATE);
+	as_content_rating_set_value (rating, "language-humor", AS_CONTENT_RATING_VALUE_MILD);
+
+	as_component_add_content_rating (cpt, rating);
+
+	/* test collection serialization */
+	res = as_yaml_test_serialize (cpt);
+	g_assert (as_test_compare_lines (res, yamldata_content_rating_field));
+}
+
+/**
+ * test_yaml_read_content_rating:
+ *
+ * Test if reading the ContentRating field works.
+ */
+static void
+test_yaml_read_content_rating (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	AsContentRating *rating;
+
+	cpt = as_yaml_test_read_data (yamldata_content_rating_field, NULL);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.ContentRatingTest");
+
+	rating = as_component_get_content_rating (cpt, "oars-1.0");
+	g_assert_nonnull (rating);
+	g_assert_cmpint (as_content_rating_get_value (rating, "drugs-alcohol"), ==, AS_CONTENT_RATING_VALUE_MODERATE);
+	g_assert_cmpint (as_content_rating_get_value (rating, "language-humor"), ==, AS_CONTENT_RATING_VALUE_MILD);
+}
+
+static const gchar *yamldata_launchable_field = "---\n"
+						"File: DEP-11\n"
+						"Version: '0.12'\n"
+						"---\n"
+						"Type: generic\n"
+						"ID: org.example.LaunchTest\n"
+						"Launchable:\n"
+						"  desktop-id:\n"
+						"  - org.example.Test.desktop\n"
+						"  - kde4-kool.desktop\n";
+
+/**
+ * test_yaml_write_launchable:
+ *
+ * Test writing the Launch field.
+ */
+static void
+test_yaml_write_launchable (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autoptr(AsLaunchable) launch = NULL;
+	g_autofree gchar *res = NULL;
+
+	cpt = as_component_new ();
+	as_component_set_kind (cpt, AS_COMPONENT_KIND_GENERIC);
+	as_component_set_id (cpt, "org.example.LaunchTest");
+
+	launch = as_launchable_new ();
+	as_launchable_set_kind (launch, AS_LAUNCHABLE_KIND_DESKTOP_ID);
+
+	as_launchable_add_entry (launch, "org.example.Test.desktop");
+	as_launchable_add_entry (launch, "kde4-kool.desktop");
+
+	as_component_add_launchable (cpt, launch);
+
+	/* test collection serialization */
+	res = as_yaml_test_serialize (cpt);
+	g_assert (as_test_compare_lines (res, yamldata_launchable_field));
+}
+
+/**
+ * test_yaml_read_launchable:
+ *
+ * Test if reading the Launch field works.
+ */
+static void
+test_yaml_read_launchable (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	AsLaunchable *launch;
+
+	cpt = as_yaml_test_read_data (yamldata_launchable_field, NULL);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.LaunchTest");
+
+	launch = as_component_get_launchable (cpt, AS_LAUNCHABLE_KIND_DESKTOP_ID);
+	g_assert_nonnull (launch);
+
+	g_assert_cmpint (as_launchable_get_entries (launch)->len, ==, 2);
+	g_assert_cmpstr (g_ptr_array_index (as_launchable_get_entries (launch), 0), ==, "org.example.Test.desktop");
+	g_assert_cmpstr (g_ptr_array_index (as_launchable_get_entries (launch), 1), ==, "kde4-kool.desktop");
+}
+
+static const gchar *yamldata_requires_recommends_field = "---\n"
+						"File: DEP-11\n"
+						"Version: '0.12'\n"
+						"---\n"
+						"Type: generic\n"
+						"ID: org.example.RelationsTest\n"
+						"Recommends:\n"
+						"- memory: '2500'\n"
+						"- modalias: usb:v1130p0202d*\n"
+						"Requires:\n"
+						"- kernel: Linux\n"
+						"  version: '>= 4.15'\n"
+						"- id: org.example.TestDependency\n"
+						"  version: == 1.2\n";
+
+/**
+ * test_yaml_write_requires_recommends:
+ *
+ * Test writing the Requires/Recommends fields.
+ */
+static void
+test_yaml_write_requires_recommends (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autofree gchar *res = NULL;
+	g_autoptr(AsRelation) mem_relation = NULL;
+	g_autoptr(AsRelation) moda_relation = NULL;
+	g_autoptr(AsRelation) kernel_relation = NULL;
+	g_autoptr(AsRelation) id_relation = NULL;
+
+	cpt = as_component_new ();
+	as_component_set_kind (cpt, AS_COMPONENT_KIND_GENERIC);
+	as_component_set_id (cpt, "org.example.RelationsTest");
+
+	mem_relation = as_relation_new ();
+	moda_relation = as_relation_new ();
+	kernel_relation = as_relation_new ();
+	id_relation = as_relation_new ();
+
+	as_relation_set_kind (mem_relation, AS_RELATION_KIND_RECOMMENDS);
+	as_relation_set_kind (moda_relation, AS_RELATION_KIND_RECOMMENDS);
+	as_relation_set_kind (kernel_relation, AS_RELATION_KIND_REQUIRES);
+	as_relation_set_kind (id_relation, AS_RELATION_KIND_REQUIRES);
+
+	as_relation_set_item_kind (mem_relation, AS_RELATION_ITEM_KIND_MEMORY);
+	as_relation_set_value (mem_relation, "2500");
+	as_relation_set_item_kind (moda_relation, AS_RELATION_ITEM_KIND_MODALIAS);
+	as_relation_set_value (moda_relation, "usb:v1130p0202d*");
+
+	as_relation_set_item_kind (kernel_relation, AS_RELATION_ITEM_KIND_KERNEL);
+	as_relation_set_value (kernel_relation, "Linux");
+	as_relation_set_version (kernel_relation, "4.15");
+	as_relation_set_compare (kernel_relation, AS_RELATION_COMPARE_GE);
+
+	as_relation_set_item_kind (id_relation, AS_RELATION_ITEM_KIND_ID);
+	as_relation_set_value (id_relation, "org.example.TestDependency");
+	as_relation_set_version (id_relation, "1.2");
+	as_relation_set_compare (id_relation, AS_RELATION_COMPARE_EQ);
+
+	as_component_add_relation (cpt, mem_relation);
+	as_component_add_relation (cpt, moda_relation);
+	as_component_add_relation (cpt, kernel_relation);
+	as_component_add_relation (cpt, id_relation);
+
+	/* test collection serialization */
+	res = as_yaml_test_serialize (cpt);
+	g_assert (as_test_compare_lines (res, yamldata_requires_recommends_field));
+}
+
+/**
+ * test_yaml_read_requires_recommends:
+ *
+ * Test if reading the Requires/Recommends fields works.
+ */
+static void
+test_yaml_read_requires_recommends (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	GPtrArray *recommends;
+	GPtrArray *requires;
+	AsRelation *relation;
+
+	cpt = as_yaml_test_read_data (yamldata_requires_recommends_field, NULL);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.RelationsTest");
+
+	recommends = as_component_get_recommends (cpt);
+	requires = as_component_get_requires (cpt);
+
+	g_assert_cmpint (recommends->len, ==, 2);
+	g_assert_cmpint (requires->len, ==, 2);
+
+	/* memory relation */
+	relation = AS_RELATION (g_ptr_array_index (recommends, 0));
+	g_assert_cmpint (as_relation_get_kind (relation), ==, AS_RELATION_KIND_RECOMMENDS);
+	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_MEMORY);
+	g_assert_cmpint (as_relation_get_value_int (relation), ==, 2500);
+
+	/* modalias relation */
+	relation = AS_RELATION (g_ptr_array_index (recommends, 1));
+	g_assert_cmpint (as_relation_get_kind (relation), ==, AS_RELATION_KIND_RECOMMENDS);
+	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_MODALIAS);
+	g_assert_cmpstr (as_relation_get_value (relation), ==, "usb:v1130p0202d*");
+
+	/* kernel relation */
+	relation = AS_RELATION (g_ptr_array_index (requires, 0));
+	g_assert_cmpint (as_relation_get_kind (relation), ==, AS_RELATION_KIND_REQUIRES);
+	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_KERNEL);
+	g_assert_cmpstr (as_relation_get_value (relation), ==, "Linux");
+	g_assert_cmpstr (as_relation_get_version (relation), ==, "4.15");
+	g_assert_cmpint (as_relation_get_compare (relation), ==, AS_RELATION_COMPARE_GE);
+
+	/* ID relation */
+	relation = AS_RELATION (g_ptr_array_index (requires, 1));
+	g_assert_cmpint (as_relation_get_kind (relation), ==, AS_RELATION_KIND_REQUIRES);
+	g_assert_cmpint (as_relation_get_item_kind (relation), ==, AS_RELATION_ITEM_KIND_ID);
+	g_assert_cmpstr (as_relation_get_value (relation), ==, "org.example.TestDependency");
+	g_assert_cmpstr (as_relation_get_version (relation), ==, "1.2");
+	g_assert_cmpint (as_relation_get_compare (relation), ==, AS_RELATION_COMPARE_EQ);
+}
+
 /**
  * main:
  */
@@ -685,6 +935,15 @@ main (int argc, char **argv)
 	g_test_add_func ("/YAML/Read/Custom", test_yaml_read_custom);
 	g_test_add_func ("/YAML/Write/Custom", test_yaml_write_custom);
 
+	g_test_add_func ("/YAML/Read/ContentRating", test_yaml_read_content_rating);
+	g_test_add_func ("/YAML/Write/ContentRating", test_yaml_write_content_rating);
+
+	g_test_add_func ("/YAML/Read/Launchable", test_yaml_read_launchable);
+	g_test_add_func ("/YAML/Write/Launchable", test_yaml_write_launchable);
+
+	g_test_add_func ("/YAML/Read/RequiresRecommends", test_yaml_read_requires_recommends);
+	g_test_add_func ("/YAML/Write/RequiresRecommends", test_yaml_write_requires_recommends);
+
 	ret = g_test_run ();
 	g_free (datadir);
 	return ret;
