@@ -54,17 +54,17 @@ const GOptionEntry data_collection_options[] = {
 		G_OPTION_ARG_STRING,
 		&optn_cachepath,
 		/* TRANSLATORS: ascli flag description for: --cachepath */
-		N_("Manually set the location of the AppStream cache."), NULL },
+		N_("Manually selected location of AppStream cache."), NULL },
 	{ "datapath", 0, 0,
 		G_OPTION_ARG_STRING,
 		&optn_datapath,
 		/* TRANSLATORS: ascli flag description for: --datapath */
-		N_("Manually set the location of AppStream metadata to scan."), NULL },
+		N_("Manually selected location of AppStream metadata to scan."), NULL },
 	{ "no-cache", 0, 0,
 		G_OPTION_ARG_NONE,
 		&optn_no_cache,
 		/* TRANSLATORS: ascli flag description for: --no-cache */
-		N_("Do not use any caches when performing the request."),
+		N_("Make request without any caching."),
 		NULL },
 	{ NULL }
 };
@@ -80,7 +80,7 @@ const GOptionEntry format_options[] = {
 		G_OPTION_ARG_STRING,
 		&optn_format,
 		/* TRANSLATORS: ascli flag description for: --format */
-		N_("Default to the given metadata format (valid values are 'xml' and 'yaml')."), NULL },
+		N_("Default metadata format (valid values are 'xml' and 'yaml')."), NULL },
 	{ NULL }
 };
 
@@ -102,6 +102,7 @@ const GOptionEntry find_options[] = {
 
 /* used by validate_options */
 static gboolean optn_pedantic = FALSE;
+static gboolean optn_nonet = FALSE;
 
 /**
  * General options for validation.
@@ -110,8 +111,17 @@ const GOptionEntry validate_options[] = {
 	{ "pedantic", (gchar) 0, 0,
 		G_OPTION_ARG_NONE,
 		&optn_pedantic,
-		/* TRANSLATORS: ascli flag description for: --pedantic */
-		N_("Also print pedantic hints when validating."), NULL },
+		/* TRANSLATORS: ascli flag description for: --pedantic (used by the "validate" command) */
+		N_("Also show pedantic hints."), NULL },
+	{ "no-net", (gchar) 0, 0,
+		G_OPTION_ARG_NONE,
+		&optn_nonet,
+		/* TRANSLATORS: ascli flag description for: --no-net (used by the "validate" command) */
+		N_("Do not use network access."), NULL },
+	{ "nonet", (gchar) 0, G_OPTION_FLAG_HIDDEN,
+		G_OPTION_ARG_NONE,
+		&optn_nonet,
+		NULL, NULL },
 	{ NULL }
 };
 
@@ -379,7 +389,8 @@ as_client_run_validate (char **argv, int argc)
 
 	return ascli_validate_files (&argv[2],
 				     argc-2,
-				     optn_pedantic);
+				     optn_pedantic,
+				     !optn_nonet);
 }
 
 /**
@@ -405,7 +416,8 @@ as_client_run_validate_tree (char **argv, int argc)
 		value = argv[2];
 
 	return ascli_validate_tree (value,
-				    optn_pedantic);
+				    optn_pedantic,
+				    !optn_nonet);
 }
 
 /**
@@ -529,6 +541,8 @@ static int
 as_client_run_new_template (char **argv, int argc)
 {
 	g_autoptr(GOptionContext) opt_context = NULL;
+	g_autoptr(GString) desc_str = NULL;
+	guint i;
 	gint ret;
 	const gchar *command = "new-template";
 	const gchar *out_fname = NULL;
@@ -540,11 +554,21 @@ as_client_run_new_template (char **argv, int argc)
 			G_OPTION_ARG_STRING,
 			&optn_desktop_file,
 			/* TRANSLATORS: ascli flag description for: --from-desktop (part of the new-template subcommand) */
-			N_("Use the given .desktop file to fill in basic values in the metainfo file."), NULL },
+			N_("Use the given .desktop file to fill in the basic values of the metainfo file."), NULL },
 		{ NULL }
 	};
 
+	/* TRANSLATORS: Additional help text for the 'new-template' ascli subcommand */
+	desc_str = g_string_new (_("This command takes optional TYPE and FILE positional arguments, FILE being a file to write to (or \"-\" for standard output)."));
+	g_string_append (desc_str, "\n");
+	/* TRANSLATORS: Additional help text for the 'new-template' ascli subcommand, a bullet-pointed list of types follows */
+	g_string_append_printf (desc_str, _("The TYPE must be a valid component-type, such as: %s"), "\n");
+	for (i = 1; i < AS_COMPONENT_KIND_LAST; i++)
+		g_string_append_printf (desc_str, " • %s\n", as_component_kind_to_string (i));
+
 	opt_context = as_client_new_subcommand_option_context (command, newtemplate_options);
+	g_option_context_set_description (opt_context, desc_str->str);
+
 	ret = as_client_option_context_parse (opt_context,
 					      command, &argc, &argv);
 	if (ret != 0)
@@ -577,7 +601,7 @@ as_client_get_summary ()
 	g_string_append_printf (string, "  %s - %s\n", "search TERM     ", _("Search the component database."));
 	g_string_append_printf (string, "  %s - %s\n", "get COMPONENT-ID", _("Get information about a component by its ID."));
 	g_string_append_printf (string, "  %s - %s\n", "what-provides TYPE VALUE", _("Get components which provide the given item."));
-	g_string_append_printf (string, "    %s - %s\n", "TYPE ", _("An item type (e.g. lib, bin, python3, ...)"));
+	g_string_append_printf (string, "    %s - %s\n", "TYPE ", _("An item type (e.g. lib, bin, python3, …)"));
 	g_string_append_printf (string, "    %s - %s\n", "VALUE", _("Value of the item that should be found."));
 	g_string_append (string, "\n");
 	g_string_append_printf (string, "  %s - %s\n", "dump COMPONENT-ID", _("Dump raw XML metadata for a component matching the ID."));
@@ -586,14 +610,14 @@ as_client_get_summary ()
 	g_string_append_printf (string, "  %s - %s\n", "validate FILE          ", _("Validate AppStream XML files for issues."));
 	g_string_append_printf (string, "  %s - %s\n", "validate-tree DIRECTORY", _("Validate an installed file-tree of an application for valid metadata."));
 	g_string_append (string, "\n");
-	g_string_append_printf (string, "  %s - %s\n", "install COMPONENT-ID", _("Install software matching the component-id."));
-	g_string_append_printf (string, "  %s - %s\n", "remove  COMPONENT-ID", _("Remove software matching the component-id."));
+	g_string_append_printf (string, "  %s - %s\n", "install COMPONENT-ID", _("Install software matching the component-ID."));
+	g_string_append_printf (string, "  %s - %s\n", "remove  COMPONENT-ID", _("Remove software matching the component-ID."));
 	g_string_append (string, "\n");
 	g_string_append_printf (string, "  %s - %s\n", "status           ", _("Display status information about available AppStream metadata."));
 	g_string_append_printf (string, "  %s - %s\n", "put FILE         ", _("Install a metadata file into the right location."));
 	/* TRANSLATORS: "convert" command in ascli. "Collection XML" is a term describing a specific type of AppStream XML data. */
 	g_string_append_printf (string, "  %s - %s\n", "convert FILE FILE", _("Convert collection XML to YAML or vice versa."));
-	g_string_append_printf (string, "  %s - %s\n", "new-template TYPE FILE", _("Create a template for a metainfo file (to be filed out by the upstream project)."));
+	g_string_append_printf (string, "  %s - %s\n", "new-template TYPE FILE", _("Create a template for a metainfo file (to be filled out by the upstream project)."));
 
 	g_string_append (string, "\n");
 	g_string_append (string, _("You can find information about subcommand-specific options by passing \"--help\" to the subcommand."));
@@ -630,7 +654,7 @@ as_client_run (char **argv, int argc)
 		{ "no-color", (gchar) 0, 0,
 			G_OPTION_ARG_NONE, &optn_no_color,
 			/* TRANSLATORS: ascli flag description for: --no-color */
-			_("Don't show colored output."), NULL },
+			_("Don\'t show colored output."), NULL },
 		{ NULL }
 	};
 
@@ -664,8 +688,13 @@ as_client_run (char **argv, int argc)
 		return ret;
 
 	if (optn_show_version) {
-		/* TRANSLATORS: Output if appstreamcli --version is executed. */
-		ascli_print_stdout (_("AppStream CLI tool version: %s"), VERSION);
+		if (g_strcmp0 (as_get_appstream_version (), PACKAGE_VERSION) == 0) {
+			/* TRANSLATORS: Output if appstreamcli --version is executed. */
+			ascli_print_stdout (_("AppStream version: %s"), PACKAGE_VERSION);
+		} else {
+			/* TRANSLATORS: Output if appstreamcli --version is run and the CLI and libappstream versions differ. */
+			ascli_print_stdout (_("AppStream CLI tool version: %s\nAppStream library version: %s"), PACKAGE_VERSION, as_get_appstream_version ());
+		}
 		return 0;
 	}
 
@@ -674,6 +703,12 @@ as_client_run (char **argv, int argc)
 		g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
 	}
 
+	/* allow disabling network access via an environment variable */
+	if (g_getenv ("AS_VALIDATE_NONET") != NULL) {
+		g_debug ("Disabling network usage: Environment variable AS_VALIDATE_NONET is set.");
+		optn_nonet = TRUE;
+	}
+
 	ascli_set_output_colored (!optn_no_color);
 
 	/* if out terminal is no tty, disable colors automatically */
@@ -712,7 +747,7 @@ as_client_run (char **argv, int argc)
 		return as_client_run_new_template (argv, argc);
 	} else {
 		/* TRANSLATORS: ascli has been run with unknown command. */
-		ascli_print_stderr (_("Command '%s' is unknown."), command);
+		ascli_print_stderr (_("Unknown command '%s'."), command);
 		return 1;
 	}
 }
