@@ -36,7 +36,7 @@ ascli_refresh_cache (const gchar *cachepath, const gchar *datapath, gboolean for
 {
 	g_autoptr(AsPool) dpool = NULL;
 	g_autoptr(GError) error = NULL;
-	gboolean ret;
+	gboolean ret = FALSE;
 
 	dpool = as_pool_new ();
 	if (datapath != NULL) {
@@ -61,7 +61,7 @@ ascli_refresh_cache (const gchar *cachepath, const gchar *datapath, gboolean for
 	} else {
 		as_pool_load (dpool, NULL, &error);
 		if (error == NULL)
-			as_pool_save_cache_file (dpool, cachepath, &error);
+			ret = as_pool_save_cache_file (dpool, cachepath, &error);
 	}
 
 	if (error != NULL) {
@@ -74,25 +74,17 @@ ascli_refresh_cache (const gchar *cachepath, const gchar *datapath, gboolean for
 	}
 
 	if (ret) {
-		/* we performed a cache refresh, check if we had errors */
-		if (error == NULL) {
-			/* TRANSLATORS: Updating the metadata cache succeeded */
-			g_print ("%s\n", _("AppStream cache update completed successfully."));
-		} else {
-			g_printerr ("%s\n", error->message);
-		}
+		/* we performed a cache refresh */
+
+		/* TRANSLATORS: Updating the metadata cache succeeded */
+		g_print ("%s\n", _("AppStream cache update completed successfully."));
 
 		/* no > 0 error code, since we updated something */
 		return 0;
 	} else {
-		/* cache wasn't updated, so the update wasn't necessary, or we have a fatal error */
-		if (error == NULL) {
-			g_print ("%s\n", _("AppStream cache update is not necessary."));
-			return 0;
-		} else {
-			g_printerr ("%s\n", error->message);
-			return 6;
-		}
+		/* cache wasn't updated, so the update wasn't necessary */
+		g_print ("%s\n", _("AppStream cache update is not necessary."));
+		return 0;
 	}
 }
 
@@ -208,7 +200,7 @@ ascli_what_provides (const gchar *cachepath, const gchar *kind_str, const gchar
 	g_autoptr(GError) error = NULL;
 
 	if (item == NULL) {
-		g_printerr ("%s\n", _("No value for the item to search for defined."));
+		g_printerr ("%s\n", _("No value for the item to search for was defined."));
 		return 1;
 	}
 
@@ -372,6 +364,15 @@ ascli_convert_data (const gchar *in_fname, const gchar *out_fname, AsFormatKind
 
 	metad = as_metadata_new ();
 	as_metadata_set_locale (metad, "ALL");
+
+	if ((g_str_has_suffix (in_fname, ".yml.gz")) ||
+	    (g_str_has_suffix (in_fname, ".yaml.gz")) ||
+	    (g_str_has_suffix (in_fname, ".yml")) ||
+	    (g_str_has_suffix (in_fname, ".yaml"))) {
+		/* if we have YAML, we also automatically assume a collection style */
+		as_metadata_set_format_style (metad, AS_FORMAT_STYLE_COLLECTION);
+	}
+
 	as_metadata_parse_file (metad,
 				infile,
 				AS_FORMAT_KIND_UNKNOWN,
@@ -545,7 +546,7 @@ ascli_create_metainfo_template (const gchar *out_fname, const gchar *cpt_kind_st
 	} else {
 		as_metadata_save_metainfo (metad, out_fname, AS_FORMAT_KIND_XML, &error);
 		if (error != NULL) {
-			ascli_print_stderr (_("Unable save the template metainfo file: %s"), error->message);
+			ascli_print_stderr (_("Unable to save the template metainfo file: %s"), error->message);
 			return 1;
 		}
 	}
